# Set/unset some environment variables
# USAGE: init_environment
init_environment() {
	# Unset variables that we do not want to import from the user environment
	## TODO: This should be handled by the functions initializing these variables instead.
	unset SOURCE_ARCHIVE_PATH
}

